:: Contains subroutines for common I/O operations.
::  
:: USAGE
:: Call any subroutine from your script with
:: CALL "path\to\number.cmd" :SUBROUTINE param1 param2 ...
:: 
:: EXAMPLE
:: CALL "%~dp0\io.cmd" :PRINT "one two" "three" "four and five"
:: 
:: REQUIREMENTS
:: - Windows 2000 and higher
:: 
:: @AUTHOR Fabio Andree Scheurel
:: @VERSION 1.0
SHIFT /1 & GOTO %~1



:: Prints all arguments into one line without new line.
::
:: @PARAM varargs variable number of arguments
:: @RETURN void
:: @SINCE 1.0
:PRINT
SETLOCAL ENABLEEXTENSIONS
(SET /P irrelevant=%1) < NUL
:PRINT_LOOP
SHIFT
IF NOT "%~1" == "" (
    (SET /P irrelevant= %1) < NUL
    GOTO :PRINT_LOOP
)
ENDLOCAL
GOTO :EOF



:: Prints all arguments into one line with delimiting new line.
:: 
:: @PARAM varargs variable number of arguments
:: @RETURN void
:: @SINCE 1.0
:PRINTLN
SETLOCAL ENABLEEXTENSIONS
(SET /P irrelevant=%1) < NUL
:PRINTLN_LOOP
SHIFT
IF NOT "%~1" == "" (
    (SET /P irrelevant= %1) < NUL
    GOTO :PRINTLN_LOOP
)
ECHO.
ENDLOCAL
GOTO :EOF



:: Tests for exisiting directory.
:: 
:: @PARAM String path to directory or file
:: @RESULT String "true" if input is an existing directory, otherwise "false"
:: @SINCE 1.0
:ISDIR
SETLOCAL ENABLEEXTENSIONS
IF EXIST "%~1" (
    IF "%~dp1" == "%~f1" (
        SET RESULT=true
    ) ELSE (
        FOR /F "tokens=2* skip=3" %%A IN ('DIR "%~1"') DO (
            IF "%%B" == "%~1" (
	        SET RESULT=true
	    ) ELSE (
	        SET RESULT=false
	    )
	    GOTO :ISDIR_INNER_LABEL
	)
    )
) ELSE (
  SET RESULT=false
)
:ISDIR_INNER_LABEL
ENDLOCAL & SET RESULT=%RESULT%
GOTO :EOF



:: Creates an empty temporary file within the user's temp directory.
::
:: @RETURN String full path to temp file
:: @SINCE 1.0
:CREATE_TEMPFILE
SETLOCAL ENABLEEXTENSIONS ENABLEDELAYEDEXPANSION
:CREATE_TEMPFILE_LOOP
SET NAME=!TEMP!\%RANDOM%!RANDOM!.tmp
IF EXIST "!NAME!" (
    GOTO :CREATE_TEMPFILE_LOOP
)
TYPE NUL > !NAME!
ENDLOCAL & SET RESULT="%NAME:"=^"%"& SET RESULT=!RESULT:~1,-1!
GOTO :EOF

