:: Contains subroutines for working with numbers.
:: 
:: USAGE
:: Call any subroutine from your script with
:: CALL "path\to\number.cmd" :SUBROUTINE param1 param2 ...
:: 
:: EXAMPLE
:: CALL %~dps0lib\number.cmd :NEAREST "0 5 10 15 20 25 30" %NUM%
::
:: REQUIREMENTS
:: - Windows 2000 and newer
:: 
:: @AUTHOR Fabio Andree Scheurel
:: @VERSION 1.0
SHIFT /1 & GOTO %~1



:: Checks if the first parameter is a number. That is it only contains the
:: characters 0 to 9 and additionally the minus as the first character.
:: 
:: @PARAM String string to test for number
:: @RETURN Boolean "true" if string is a valid number, "false" otherwise
:: @SINCE 1.0
:IS_NUMBER
SETLOCAL ENABLEEXTENSIONS
FOR /F "tokens=*" %%A IN ('SET /A %~1 2^>^&1') DO (
    IF "%~1" == "%%A" (
       SET RESULT=true
    ) ELSE (
       SET RESULT=false
    )
)
ENDLOCAL & SET RESULT=%RESULT%
GOTO :EOF



:: Returns the nearest matching number from a sorted list of numbers.
:: 
:: @PARAM Integer[] list of integers sorted ascending
:: @PARAM Integer number to match
:: @RETURN Integer nearest matching number of list
:: @SINCE 1.0
:NEAREST
SETLOCAL ENABLEEXTENSIONS ENABLEDELAYEDEXPANSION
SET RESULT=
FOR %%A IN (%~1) DO (
    IF "!RESULT!" == "" (
        SET RESULT=%%A
    )
    IF %%A LEQ %~2 (
        SET RESULT=%%A
	IF %%A EQU %~2 (
            GOTO :NEAREST_INNER_LABEL
	)
    ) ELSE (
        IF NOT "!RESULT!" == "%%A" (
	    SET /A DIFF1 = %~2 - !RESULT!
	    SET /A DIFF2 = %%A - %~2
	    IF !DIFF2! LSS !DIFF1! (
	        SET RESULT=%%A
	    )
	)
        GOTO :NEAREST_INNER_LABEL
    )
)
:NEAREST_INNER_LABEL
ENDLOCAL & SET RESULT="%RESULT%"& SET RESULT=!RESULT:~1,-1!
GOTO :EOF



:: Returns the nearest matching number from a sorted list of numbers that is
:: equal or less. If no such number can be found, the smallest from the list
:: will be returned.
:: 
:: @PARAM Integer[] list of integers sorted ascending
:: @PARAM Integer number to match
:: @RETURN Integer nearest matching number of list
:: @SINCE 1.0
:NEAREST_LESS
SETLOCAL ENABLEEXTENSIONS ENABLEDELAYEDEXPANSION
SET RESULT=
FOR %%A IN (%~1) DO (
    IF "!RESULT!" == "" (
        SET RESULT=%%A
    ) 
    IF %%A LEQ %~2 (
        SET RESULT=%%A
    ) ELSE (
        GOTO :NEAREST_LESS_INNER_LABEL
    )
)
:NEAREST_LESS_INNER_LABEL
ENDLOCAL & SET RESULT="%RESULT%"& SET RESULT=!RESULT:~1,-1!
GOTO :EOF



:: Returns the nearest matching number from a sorted list of numbers that is
:: equal or greater. If no such number can be found, the greatest from the list
:: will be returned.
:: 
:: @PARAM Integer[] list of integers sorted ascending
:: @PARAM Integer number to match
:: @RETURN Integer nearest matching number of list
:: @SINCE 1.0
:NEAREST_GREATER
SETLOCAL ENABLEEXTENSIONS ENABLEDELAYEDEXPANSION
SET RESULT=
FOR %%A IN (%~1) DO (
    IF "!RESULT!" == "" (
        SET RESULT=%%A
    ) 
    IF %%A GEQ %~2 (
        GOTO :NEAREST_GREATER_INNER_LABEL
    ) ELSE (
        SET RESULT=%%A
    )
)
:NEAREST_GREATER_INNER_LABEL
ENDLOCAL & SET RESULT="%RESULT%"& SET RESULT=!RESULT:~1,-1!
GOTO :EOF



:: Converts a decimal number to hex. Currently only numbers 0 <= x <= 255 are 
:: supported (0x00 <= x <= 0xFF). The result will NOT be prefixed with 0x!
:: 
:: @PARAM Integer decimal number 0 <= x <= 255
:: @RETURN String hex representation of number
:: @SINCE 1.0
:DEC2HEX
SETLOCAL ENABLEEXTENSIONS ENABLEDELAYEDEXPANSION
SET HEX=0123456789ABCDEF
SET /A T1=%1 / 16
SET /A T2=%1 %% 16
SET RESULT=!HEX:~%T1%,1!!HEX:~%T2%,1!
ENDLOCAL & SET RESULT=%RESULT%
GOTO :EOF

