:: Contains subroutines for working with strings.
:: 
:: USAGE
:: Call any subroutine from your script with
:: CALL "path\to\number.cmd" :SUBROUTINE param1 param2 ...
:: 
:: EXAMPLE
:: CALL "%~dp0\string.cmd" :STRLEN "how long is this string?"
::
:: REQUIREMENTS
:: - Windows 2000 and higher
:: 
:: @AUTHOR Fabio Andree Scheurel
:: @VERSION 1.0
SHIFT /1 & GOTO %~1



:: Returns the string length.
:: 
:: @PARAM String string
:: @RETURN Integer string length
:: @SINCE 1.0
:STRLEN
SETLOCAL ENABLEEXTENSIONS ENABLEDELAYEDEXPANSION
SET STR="%~1"& SET STR=!STR:~1,-1!
SET LEN=0
:: Max string length is 8191, see http://support.microsoft.com/kb/830473
SET POS=8192
:STRLEN_LOOP
SET /A POS /= 2
IF NOT "!STR!" == "" (
    IF NOT "!STR:~%POS%,1!" == "" (
        SET /A LEN += %POS% + 1
        :: work on rest, that is after POS+1
        SET STR=!STR:~%POS%!
        SET STR=!STR:~1!
    )
    GOTO :STRLEN_LOOP
)
ENDLOCAL & SET RESULT=%LEN%
GOTO :EOF



:: Extracts a substring. Works exactly the same the string substitution
:: function %VAR:~x,y% does. Provides a way for dynamically extracting
:: substrings.
:: 
:: @PARAM String string to extract substring of
:: @PARAM Integer x; 0 or greater: string offset, negative number: string offset
:: from end
:: @PARAM Integer?-0 y; 0 or greater: number of characters to extract, negative
:: number: string offset from end where to stop substring extraction
:: @RETURN String substring
:: @SINCE 1.0
:SUBSTR
SETLOCAL ENABLEEXTENSIONS ENABLEDELAYEDEXPANSION
SET RESULT="%~1"
SET RESULT=!RESULT:~1,-1!
IF NOT "%~3" == "" (
    SET RESULT=!RESULT:~%~2,%~3!
) ELSE (
    SET RESULT=!RESULT:~%~2!
)
SET RESULT= !RESULT!
ENDLOCAL & SET RESULT="%RESULT:"=^"%"& SET RESULT=!RESULT:~2,-1!
GOTO :EOF



:: Trims whitespace at beginning and end of a string.
:: 
:: @PARAM String string to trim
:: @RETURN String trimmed string
:: @SINCE 1.0
:TRIM
SETLOCAL ENABLEEXTENSIONS ENABLEDELAYEDEXPANSION
SET STR="%~1"& SET STR=!STR:~1,-1!
:: whitespace is space and tab
SET WHITESPACE= 	
:TRIM_INNER_LOOP
IF "!STR!" NEQ "" (
    IF "!WHITESPACE:%STR:~0,1%=!" NEQ "!WHITESPACE!" (
        SET STR=!STR:~1!
        GOTO :TRIM_INNER_LOOP
    ) ELSE IF "!WHITESPACE:%STR:~-1%=!" NEQ "!WHITESPACE!" (
        SET STR=!STR:~0,-1!
        GOTO :TRIM_INNER_LOOP
    )
)
ENDLOCAL & SET RESULT="%STR%"& SET RESULT=!RESULT:~1,-1!
GOTO :EOF



:: Returns a string containing all ASCII characters at their given offset.
:: All (in batch) non-printable characters will be replaced by a space.
:: 
:: @RETURN String all ASCII characters
:: @SINCE 1.0
:GET_ASCII_CHARS
SET RESULT=         	                       ^^!"#$%%&'()*+,-./0123456789.;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^^_`abcdefghijklmnopqrstuvwxyz{|}~ 
GOTO :EOF



:: Converts the ASCII string to a space separated list of hex numbers.
:: For example "ABC" will be converted to "41 42 43".
::
:: @PARAM String ASCII string
:: @PARAM Boolean?false use 0x prefix
:: @PARAM String?" " separator string
:: @RETURN String[] list of hex numbers
:: @SINCE 1.0
:ASC2HEX
SETLOCAL ENABLEEXTENSIONS ENABLEDELAYEDEXPANSION
SET INPUT="%~1"& SET INPUT=!INPUT:~1,-1!
IF "%~2" == "true" (
  SET PREFIX=0x
)
IF "%~3" == "" (
  SET SEPARATOR= 
) ELSE (
  SET SEPARATOR="%~3"& SET SEPARATOR=!SEPARATOR:~1,-1!
)
CALL :ASC2DEC "!INPUT!" ","
SET DEC=!RESULT!
SET HEX=
:ASC2HEX_DECIMAL_LOOP
FOR /F "tokens=1* delims=," %%I IN ("!DEC!") DO (
  CALL %~dps0number.cmd :DEC2HEX %%I
  IF "!HEX!" == "" (
    SET HEX=!PREFIX!!RESULT!
  ) ELSE (
    SET HEX=!HEX!!SEPARATOR!!PREFIX!!RESULT!
  )
  SET DEC=%%J
  GOTO :ASC2HEX_DECIMAL_LOOP
)
ENDLOCAL & SET RESULT=%HEX%
GOTO :EOF



:: Converts an ASCII string to a space separated list of decimal numbers. For 
:: each character in the the string it's decimal ASCII index will be used.
:: For example the string "ABC" will be converted to "65 66 67".
:: 
:: @PARAM String ASCII string
:: @PARAM String?" " list item separator string
:: @RETURN Integer[] list of decimals
:: @SINCE 1.0
:ASC2DEC
SETLOCAL ENABLEEXTENSIONS ENABLEDELAYEDEXPANSION
SET INPUT="%~1"& SET INPUT=!INPUT:~1,-1!
IF "%~2" == "" (
  SET SEPARATOR= 
) ELSE (
  SET SEPARATOR="%~2"& SET SEPARATOR=!SEPARATOR:~1,-1!
)
SET OUTPUT=
SET CHAR=
CALL :STRLEN "!INPUT!"
SET /A MAX=!RESULT! - 1
FOR /L %%I IN (0,1,!MAX!) DO (
  SET CHAR=!INPUT:~%%I,1!
  CALL :ASCII !CHAR!
  IF !RESULT! == 0 (
    ECHO WARNING: Character '!CHAR!' is not a valid ASCII character. Ignoring... >&2
  ) ELSE IF "!OUTPUT!" == "" (
    SET OUTPUT=!RESULT!
  ) ELSE (
    SET OUTPUT=!OUTPUT!!SEPARATOR!!RESULT!
  )
)
ENDLOCAL & SET RESULT=%OUTPUT%
GOTO :EOF



:: Converts a list of hexadecimal numbers to an ASCII string. Automatically
:: detects if numbers are prefixed with "0x".
:: 
:: @PARAM String[] hexadecimal list
:: @PARAM String?" " list item separator
:: @RETURN String ASCII string
:: @SINCE 1.0
:HEX2ASC
SETLOCAL ENABLEEXTENSIONS ENABLEDELAYEDEXPANSION
SET INPUT="%~1"& SET INPUT=!INPUT:~1,-1!
IF "%~2" == "" (
  SET SEPARATOR= 
) ELSE (
  SET SEPARATOR="%~2"& SET SEPARATOR=!SEPARATOR:~1,-1!
)
SET HEX=!INPUT!
SET ASC=
SET DEC=
:HEX2ASC_DECIMAL_LOOP
FOR /F "tokens=1* delims=%SEPARATOR%" %%I IN ("!HEX!") DO (
  SET I2=%%I
  IF "!I2:~0,2!" == "0x" (
    SET /A DEC=!I2!
  ) ELSE (
    SET /A DEC=0x!I2!
  )
  CALL :CHAR !DEC!
  IF "!ASC!" == "" (
    SET ASC="!RESULT!"
  ) ELSE (
    SET ASC="!ASC!!RESULT!"
  )
  SET ASC=!ASC:~1,-1!
  SET HEX="%%J"& SET HEX=!HEX:~1,-1!
  GOTO :HEX2ASC_DECIMAL_LOOP
)
ENDLOCAL & SET RESULT="%ASC%"& SET RESULT=!RESULT:~1,-1!
GOTO :EOF



:: Converts a list of decimal numbers to an ASCII string.
:: 
:: @PARAM Integer[] decimal list
:: @PARAM String?" " list item separator string
:: @RETURN String ASCII string
:: @SINCE 1.0
:DEC2ASC
SETLOCAL ENABLEEXTENSIONS ENABLEDELAYEDEXPANSION
SET INPUT="%~1"& SET INPUT=!INPUT:~1,-1!
IF "%~2" == "" (
  SET SEPARATOR= 
) ELSE (
  SET SEPARATOR="%~2"& SET SEPARATOR=!SEPARATOR:~1,-1!
)
SET DEC=!INPUT!
SET ASC=
:DEC2ASC_DECIMAL_LOOP
FOR /F "tokens=1* delims=%SEPARATOR%" %%I IN ("!DEC!") DO (
  CALL :CHAR %%I
  IF "!ASC!" == "" (
    SET ASC="!RESULT!"
  ) ELSE (
    SET ASC="!ASC!!RESULT!"
  )
  SET ASC=!ASC:~1,-1!
  SET DEC="%%J"& SET DEC=!DEC:~1,-1!
  GOTO :DEC2ASC_DECIMAL_LOOP
)
ENDLOCAL & SET RESULT="%ASC%"& SET RESULT=!RESULT:~1,-1!
GOTO :EOF



:: Returns the numeric (decimal) ASCII code of the character.
:: 
:: @PARAM String single character
:: @RETURN Integer ASCII code
:: @SINCE 1.0
:ASCII
SETLOCAL ENABLEEXTENSIONS ENABLEDELAYEDEXPANSION
SET CHAR="%~1"& SET CHAR=!CHAR:~1,1!
IF "!CHAR!" == "" (
  SET CODE=0
) ELSE IF "!CHAR!" == " " (
  SET CODE=32
) ELSE IF "!CHAR!" == "	" (
  SET CODE=9
) ELSE (
  CALL :GET_ASCII_CHARS
  SET ASCII=!RESULT!
  FOR /L %%J IN (33,1,126) DO (
    IF "!CHAR!" == "!ASCII:~%%J,1!" (
      SET CODE=%%J
      GOTO :ASCII_RETURN_RESULT
    )
  )
  SET CODE=0
)
:ASCII_RETURN_RESULT
ENDLOCAL & SET RESULT=%CODE%
GOTO :EOF



:: Returns the ASCII character denoted by the decimal number. Non-printable 
:: characters will be returned as space.
:: 
:: @PARAM Integer ASCII offset number of character
:: @RETURN String ASCII character
:: @SINCE 1.0
:CHAR
SETLOCAL ENABLEEXTENSIONS ENABLEDELAYEDEXPANSION
CALL :GET_ASCII_CHARS
SET RESULT="!RESULT:~%~1,1!"
IF !RESULT! == "^!" (
  ENDLOCAL & SET RESULT=^^!
  GOTO :EOF
) ELSE IF !RESULT! == "&" (
  SET RESULT=!RESULT:^&=^^^&!
) ELSE IF !RESULT! == "<" (
  SET RESULT=!RESULT:^<=^^^<!
) ELSE IF !RESULT! == ">" (
  SET RESULT=!RESULT:^>=^^^>!
) ELSE IF !RESULT! == "^" (
  SET RESULT=!RESULT:^^=^^^^!
) ELSE IF !RESULT! == "|" (
  SET RESULT=!RESULT:^|=^^^|!
)
ENDLOCAL & SET RESULT=%RESULT:~1,-1%
GOTO :EOF
